@@ -217,6 +217,9 @@ class OopStorage;
   do_klass(Integer_klass,                               java_lang_Integer,                         Pre                 ) \
   do_klass(Long_klass,                                  java_lang_Long,                            Pre                 ) \
                                                                                                                          \
+  /* force inline of iterators */                                                                                        \
+  do_klass(Iterator_klass,                              java_util_Iterator,                        Pre                 ) \
+                                                                                                                         \
   /* JVMCI classes. These are loaded on-demand. */                                                                       \
   JVMCI_WK_KLASSES_DO(do_klass)                                                                                          \
                                                                                                                          \
@@ -467,6 +470,8 @@ public:
   static bool is_well_known_klass(Symbol* class_name);
 #endif
 
+  static bool update_well_known_klass(InstanceKlass* new_klass, InstanceKlass* old_klass);
+
   // Enhanced class redefinition
   static void remove_from_hierarchy(InstanceKlass* k);
   static void update_constraints_after_redefinition();
