@@ -221,7 +221,7 @@ StringDedupTable*        StringDedupTable::_resized_table = NULL;
 StringDedupTable*        StringDedupTable::_rehashed_table = NULL;
 volatile size_t          StringDedupTable::_claimed_index = 0;
 
-StringDedupTable::StringDedupTable(size_t size, jint hash_seed) :
+StringDedupTable::StringDedupTable(size_t size, uint64_t hash_seed) :
   _size(size),
   _entries(0),
   _grow_threshold((uintx)(size * _grow_load_factor)),
@@ -234,7 +234,7 @@ StringDedupTable::StringDedupTable(size_t size, jint hash_seed) :
 }
 
 StringDedupTable::~StringDedupTable() {
-  FREE_C_HEAP_ARRAY(G1StringDedupEntry*, _buckets);
+  FREE_C_HEAP_ARRAY(StringDedupEntry*, _buckets);
 }
 
 void StringDedupTable::create() {
@@ -327,7 +327,7 @@ unsigned int StringDedupTable::hash_code(typeArrayOop value, bool latin1) {
     if (use_java_hash()) {
       hash = java_lang_String::hash_code(data, length);
     } else {
-      hash = AltHashing::murmur3_32(_table->_hash_seed, data, length);
+      hash = AltHashing::halfsiphash_32(_table->_hash_seed, (const uint8_t*)data, length);
     }
   } else {
     length /= sizeof(jchar) / sizeof(jbyte); // Convert number of bytes to number of chars
@@ -335,7 +335,7 @@ unsigned int StringDedupTable::hash_code(typeArrayOop value, bool latin1) {
     if (use_java_hash()) {
       hash = java_lang_String::hash_code(data, length);
     } else {
-      hash = AltHashing::murmur3_32(_table->_hash_seed, data, length);
+      hash = AltHashing::halfsiphash_32(_table->_hash_seed, (const uint16_t*)data, length);
     }
   }
 
@@ -655,6 +655,6 @@ void StringDedupTable::print_statistics() {
             _table->_entries, percent_of((size_t)_table->_entries, _table->_size), _entry_cache->size(), _entries_added, _entries_removed);
   log.debug("    Resize Count: " UINTX_FORMAT ", Shrink Threshold: " UINTX_FORMAT "(" STRDEDUP_PERCENT_FORMAT_NS "), Grow Threshold: " UINTX_FORMAT "(" STRDEDUP_PERCENT_FORMAT_NS ")",
             _resize_count, _table->_shrink_threshold, _shrink_load_factor * 100.0, _table->_grow_threshold, _grow_load_factor * 100.0);
-  log.debug("    Rehash Count: " UINTX_FORMAT ", Rehash Threshold: " UINTX_FORMAT ", Hash Seed: 0x%x", _rehash_count, _rehash_threshold, _table->_hash_seed);
+  log.debug("    Rehash Count: " UINTX_FORMAT ", Rehash Threshold: " UINTX_FORMAT ", Hash Seed: " UINT64_FORMAT, _rehash_count, _rehash_threshold, _table->_hash_seed);
   log.debug("    Age Threshold: " UINTX_FORMAT, StringDeduplicationAgeThreshold);
 }
