@@ -49,7 +49,6 @@ ShenandoahControlThread::ShenandoahControlThread() :
   reset_gc_id();
   create_and_start();
   _periodic_task.enroll();
-  _periodic_satb_flush_task.enroll();
   if (ShenandoahPacing) {
     _periodic_pacer_notify_task.enroll();
   }
@@ -64,10 +63,6 @@ void ShenandoahPeriodicTask::task() {
   _thread->handle_counters_update();
 }
 
-void ShenandoahPeriodicSATBFlushTask::task() {
-  ShenandoahHeap::heap()->force_satb_flush_all_threads();
-}
-
 void ShenandoahPeriodicPacerNotify::task() {
   assert(ShenandoahPacing, "Should not be here otherwise");
   ShenandoahHeap::heap()->pacer()->notify_waiters();
