@@ -78,9 +78,10 @@ oop ResolvedMethodTable::lookup(int index, unsigned int hash, Method* method) {
 }
 
 unsigned int ResolvedMethodTable::compute_hash(Method* method) {
-  unsigned int name_hash = method->name()->identity_hash();
-  unsigned int signature_hash = method->signature()->identity_hash();
-  return name_hash ^ signature_hash;
+  unsigned int hash = method->klass_name()->identity_hash();
+  hash = (hash * 31) ^ method->name()->identity_hash();
+  hash = (hash * 31) ^ method->signature()->identity_hash();
+  return hash;
 }
 
 
