@@ -1280,8 +1280,8 @@ class JavaThread: public Thread {
     _handshake.process_by_self(this);
   }
 
-  void handshake_process_by_vmthread() {
-    _handshake.process_by_vmthread(this);
+  HandshakeState::ProcessResult handshake_try_process_by_vmThread(HandshakeOperation* op) {
+    return _handshake.try_process_by_vmThread(this);
   }
 
   // Suspend/resume support for JavaThread
@@ -2257,13 +2257,6 @@ class Threads: AllStatic {
   static void deoptimized_wrt_marked_nmethods();
 };
 
-
-// Thread iterator
-class ThreadClosure: public StackObj {
- public:
-  virtual void do_thread(Thread* thread) = 0;
-};
-
 class SignalHandlerMark: public StackObj {
  private:
   Thread* _thread;
