@@ -185,7 +185,11 @@ template<class E> class GrowableArray : public GenericGrowableArray {
     for (; i < _max; i++) ::new ((void*)&_data[i]) E();
   }
 
-  GrowableArray(Arena* arena, int initial_size, int initial_len, const E& filler) : GenericGrowableArray(arena, initial_size, initial_len) {
+  // Watch out, if filler was generated by a constructor, the destuctor might
+  // be called on the original object invalidating all the copies made here.
+  // Carefully design the copy constructor.
+  GrowableArray(Arena* arena, int initial_size, int initial_len, const E& filler) :
+      GenericGrowableArray(arena, initial_size, initial_len) {
     _data = (E*)raw_allocate(sizeof(E));
     int i = 0;
     for (; i < _len; i++) ::new ((void*)&_data[i]) E(filler);
@@ -528,6 +532,10 @@ template<class E> class GrowableArrayIterator : public StackObj {
     assert(_array == rhs._array, "iterator belongs to different array");
     return _position != rhs._position;
   }
+
+  bool at_end() { return _position >= _array->length(); }
+
+  bool has_next() { return _position < _array->length() - 1; }
 };
 
 // Custom STL-style iterator to iterate over elements of a GrowableArray that satisfy a given predicate
