@@ -211,6 +211,11 @@ int isOptSep(char c) {
   *
   */
 
+#if !defined(__clang_major__) && defined(__GNUC__) && (__GNUC__ >= 8)
+_Pragma("GCC diagnostic push")
+_Pragma("GCC diagnostic ignored \"-Wstringop-truncation\"")
+#endif
+
 int nsk_jvmti_parseOptions(const char options[]) {
     size_t len;
     const char* opt;
@@ -291,6 +296,9 @@ int nsk_jvmti_parseOptions(const char options[]) {
     return success;
 }
 
+#if !defined(__clang_major__) && defined(__GNUC__) && (__GNUC__ >= 8)
+_Pragma("GCC diagnostic pop")
+#endif
 
 /*************************************************************/
 
